<?php
define('lnmasker', '__BR__');

class TurboWeb2Print {
    private $designerpath = '';
    private $templatefolder = '';
    private $templatePath = '';
    private $xml = '';
    public $documentName = '';
    private $imageMagickPath = '';
    public $xmlVals = array();
    private $session = array();

    // errechnet sich aus 240 DPI durch 25,4
    var $inch2pixel = 25.4;

    function __construct($templatefolder, $altPath='', $sessionArr=array()) {
        $this->session =  $sessionArr;
        $templatefolder = 'private/'.$templatefolder;
        if (!empty($altPath)) {
            $altPath = preg_replace('/^\/+/', '', preg_replace('/\/+$/', '', $altPath)).'/';
            if (file_exists($altPath)) {
                // do nothing
            } elseif(file_exists(getcwd().'/'.$altPath)) {
                $altPath = getcwd().'/'.$altPath;
            } else {
                $altPath = '';
            }
        }
        if (empty($altPath)) {
            $designerpath = getcwd().'/fileadmin/designer/documentLibrary/';
        } else {
            $designerpath = $altPath.'documentLibrary/';
        }
        $this->templatePath = $this->addPath($designerpath, $templatefolder);
        if (is_dir($this->templatePath)) {
            $this->designerpath = $designerpath;
            $this->templatefolder = $templatefolder;
        } else {
            return 0;
        }
        if (empty($altPath)) {
            @require_once(getcwd().'/fileadmin/designer/server/libraries/constants.inc.php');
            require_once(getcwd().'/fileadmin/designer/server/configuration.php');
        } else {
            @require_once($altPath.'server/libraries/constants.inc.php');
            require_once($altPath.'server/configuration.php');
        }
        if (defined('IMAGEMAGICK_PATH')) {
            $this->imageMagickPath = IMAGEMAGICK_PATH;
            if (substr($this->imageMagickPath, -1) != '/') $this->imageMagickPath .= '/';
        } else {
            return 0;
        }
    }

    private function addPath($a1, $a2) {
        return str_replace('//', '/', $a1 . '/' . $a2);
    }

    function getDirectoryList() {
        $dirs = array();
        $d = dir($this->templatePath);
        while (false !== ($entry = $d->read())) {
            if ($entry == '.' || $entry== '..') continue;

            if (!$this->getDocument($entry)) continue;
            $dirs[] = $entry;
        }
        $d->close();
        return $dirs;
    }

    function getDocument($documentname) {
        if (empty($this->templatefolder)) return 0;
        $this->documentName = $documentname;
        $documentname = $this->addPath($documentname, '.structure.xml');
        $path = $this->addPath($this->templatePath, $documentname);
        if (file_exists($path))	{
            $this->xml = file_get_contents($path);
            $pattern = '@\<\?php(.*)\?\>@iU';
            $this->xml = str_replace("\n", lnmasker, $this->xml);
            if ($result = preg_match_all($pattern, $this->xml, $subpattern)) {
                foreach ($subpattern[1] as $key => $exec) {
                    ob_start();
                    eval(str_replace(lnmasker, "\n", $exec));
                    $ret = ob_get_contents();
                    ob_end_clean();
                    if ($debug) {
                        echo str_replace(lnmasker, "\n", $exec);
                        eval(str_replace(lnmasker, "\n", $exec));
                    }
                    $this->xml = str_replace($subpattern[0][$key], $ret, $this->xml);
                }
            }
            $this->xml = str_replace(lnmasker, "\n", $this->xml);

            $p = xml_parser_create();
            xml_parser_set_option( $p, XML_OPTION_CASE_FOLDING, 0 );
            xml_parse_into_struct($p,  $this->xml, $this->xmlVals, $index);
            xml_parser_free($p);
            return 1;
        } else {
            return 0;
        }
        $this->documentName = '';
        return 0;
    }

    function str_insert($insertstring, $intostring, $offset) {
        $part1 = substr($intostring, 0, $offset);
        $part2 = substr($intostring, $offset);
        $part1 = $part1 . $insertstring;
        $whole = $part1 . $part2;
        return $whole;
    }


    function proportional($width_old, $height_old, $width = 0, $height = 0) {
        $final_width = 0;
        $final_height = 0;
        if ($width == 0) {
            $factor = $height/$height_old;
        } elseif ($height == 0) {
            $factor = $width/$width_old;
        } else {
            $factor = min($width / $width_old, $height / $height_old);
        }
        $final_width = ($width_old * $factor);
        $final_height = ($height_old * $factor);
        return array($final_width, $final_height);
    }

    function getTag($name, $vals) {
        $str = '<'.$name;
        foreach ($vals as $keys => $v) {
            $str .= ' '.$keys.'="'.$v.'"';
        }
        return $str .= '>';
    }

    function alignPos(&$val, $oldval) {
        if (!empty($val['horizontalAlign'])) {
            // center
            if ($val['horizontalAlign'] == 1) $val['x'] += ($oldval['width']/2)-($val['width']/2);
            if ($val['horizontalAlign'] == 2) $val['x'] += ($oldval['width'])-($val['width']);
        }
        if (!empty($val['verticalAlign'])) {
            // center
            if ($val['verticalAlign'] == 1) $val['y'] += ($oldval['height']/2)-($val['height']/2);
            if ($val['verticalAlign'] == 2) $val['y'] += ($oldval['height'])-($val['height']);
        }
    }

    function getAtr($name) {
        foreach($this->xmlVals as $v) {
            if ($v['attributes']['name'] == $name) {
                return $v['attributes'];
            }
        }
    }

    function get($type, $val, $saveunder='') {
        $retArr = array();
        foreach($this->xmlVals as $v) {
            if (strtolower($v[$type]) == strtolower($val)) {
                if (!$saveunder) {
                    $retArr[] =  $v;
                } else {
                    $retArr[$v['attributes'][$saveunder]] = $v;
                }
            }
        }
        return $retArr;
    }

    function getNewBoxSize($w, $h, $tw, $th) {
        list ($nwidth, $nheight) = $this->proportional($w, $h, $tw, $th);
        $cellXminPixel = $nwidth * PDF_DENSITY / $this->inch2pixel;
        $cellYminPixel = $nheight * PDF_DENSITY / $this->inch2pixel;
        return array(round($cellXminPixel), round($cellYminPixel));
    }


    function setContents($hash = array()) {
        if (!$this->documentName) return -1;
        if (count($hash)) {
            foreach($hash as $key => $val) {
                $pattern = '@\<\?php(.*)\?\>@iU';
                $replace = '';
                $val = preg_replace($pattern, $replace, $val);

                $patter = '{'.$key.'}';
                $this->xml = str_replace($patter, $val, $this->xml);

                $cond_patter = ' dependsOnField="'.$key.'"';
                $repl = '';
                if (!$val) {
                    $repl = 'CondFail';
                }
                $this->xml = str_replace($cond_patter, $repl, $this->xml);

                $cond_patter = ' dependsOnField="!'.$key.'"';
                $repl = '';
                if ($val) {
                    $repl = 'CondFail';
                }
                $this->xml = str_replace($cond_patter, $repl, $this->xml);

                foreach($this->xmlVals as $v) {
                    if (strtolower($v['tag']) == 'dummybox') {
                        if ($v['attributes']['name'] == $key) {
                            if (file_exists($val)) {
                                $retval = array();
                                if (exec($this->imageMagickPath.'identify -format "%w %h" '.escapeshellcmd($val), $retval)) {
                                    list($w, $h) = explode(' ', $retval[0]);
                                    $newCell = $v['attributes'];
                                    list ($newCell['width'], $newCell['height']) = $this->proportional($w, $h, $v['attributes']['width'], $v['attributes']['height']);

                                    $this->alignPos($newCell, $v['attributes']);
                                    if (empty($v['attributes']['filePath'])) {
                                        $newCell['filePath'] = basename($val);
                                    } else {
                                        $newCell['filePath'] = $this->addPath($v['attributes']['filePath'], basename($val));
                                    }
                                    $this->xml = preg_replace('@<dummybox .*name="'.preg_quote($key).'".*/>@isU', $this->getTag('imageBox', $newCell), $this->xml);
                                }
                            } elseif ($val) {
                                throw new RuntimeException(
                                    'TurboWeb Fatal Error: File \''.pathinfo($val, PATHINFO_BASENAME).'\' not found!',
                                    9900000012
                                );
                            }
                        }
                    }
                }

            }
        }
        $pattern = '@\<\?lastphp(.*)\?\>@iU';
        $this->xml = str_replace("\n", lnmasker, $this->xml);
        if ($result = preg_match_all($pattern, $this->xml, $subpattern)) {
            foreach ($subpattern[1] as $key => $exec) {
                ob_start();
                eval(str_replace(lnmasker, "\n", $exec));
                $ret = ob_get_contents();
                ob_end_clean();
                if ($debug) {
                    echo str_replace(lnmasker, "\n", $exec);
                    eval(str_replace(lnmasker, "\n", $exec));
                }
                $this->xml = str_replace($subpattern[0][$key], $ret, $this->xml);
            }
        }
        $this->xml = str_replace(lnmasker, "\n", $this->xml);

        $pattern = '@{.*}@isU';
        $replace = '\1';
        $this->xml = preg_replace($pattern, $replace,  $this->xml);

        return 1;
    }

    function saveOrderDocument($newname, $imagefiles=array()) {
        if (empty($this->templatefolder)) return 0;
        if ($newname == 'base' || $newname == '.') return 0;
        if (!$newname || !$this->xml) return -2;
        if (!$this->documentName) return -1;
        $srcPath = $this->addPath($this->templatePath, $this->documentName);
        $desPat =  $this->addPath($this->designerpath, 'public/finished/'.$newname);
        if (is_dir($desPat)) exec('rm -rf "'.escapeshellcmd($desPat).'"');
        exec('cp -rf "'.escapeshellcmd($srcPath).'" "'.escapeshellcmd($desPat).'"');
        $written = file_put_contents($this->addPath($desPat, '.structure.xml'), $this->xml);
        if ($written) {
            if (count($imagefiles)) {
                foreach ($imagefiles as $path) {
                    $base = basename($path);
                    if (is_file($path)) {
                        copy($path, $this->addPath($desPat, $base));
                    }
                }

            }
        }
        return $written;
    }
}